Package: mpoly
Type: Package
Title: Symbolic Computation and More with Multivariate Polynomials
Version: 1.1.2
URL: https://github.com/dkahle/mpoly
BugReports: https://github.com/dkahle/mpoly/issues
Authors@R: person("David", "Kahle", email = "david@kahle.io", role = c("aut", "cre"), comment = c(ORCID = "0000-0002-9999-1558"))
Description: Symbolic computing with multivariate polynomials in R.
Imports: stringr (>= 1.0.0), stringi, partitions, plyr, stats, ggplot2,
        polynom, orthopolynom, tidyr
Suggests: testthat (>= 2.1.0), magrittr, dplyr, covr
License: GPL-2
RoxygenNote: 7.3.2
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2025-06-09 20:40:29 UTC; david_kahle
Author: David Kahle [aut, cre] (ORCID: <https://orcid.org/0000-0002-9999-1558>)
Maintainer: David Kahle <david@kahle.io>
Repository: CRAN
Date/Publication: 2025-06-09 21:20:02 UTC
Built: R 4.3.3; ; 2025-06-19 05:06:11 UTC; unix
