# Generated by roxygen2: do not edit by hand

S3method("!=",mpoly)
S3method("*",mpoly)
S3method("*",mpolyList)
S3method("+",mpoly)
S3method("+",mpolyList)
S3method("-",mpoly)
S3method("-",mpolyList)
S3method("==",mpoly)
S3method("[",mpoly)
S3method("^",mpoly)
S3method("^",mpolyList)
S3method(as.mpoly,alm)
S3method(as.mpoly,default)
S3method(as.mpoly,lm)
S3method(as.mpoly,numeric)
S3method(as.mpoly,polynomial)
S3method(base::as.function,bernstein)
S3method(base::as.function,bezier)
S3method(base::as.function,mpoly)
S3method(base::as.function,mpolyList)
S3method(base::plot,mpoly)
S3method(coef,mpoly)
S3method(deriv,mpoly)
S3method(print,mpoly)
S3method(print,mpolyList)
S3method(reorder,mpoly)
S3method(round,mpoly)
S3method(terms,mpoly)
export(LC)
export(LCM)
export(LM)
export(LT)
export(as.mpoly)
export(basis_monomials)
export(bernstein)
export(bernsteinApprox)
export(bernstein_approx)
export(bezier)
export(bezierFunction)
export(bezier_function)
export(burst)
export(chebyshev)
export(chebyshev_roots)
export(coef_lift)
export(dehomogenize)
export(eq_mp)
export(exponents)
export(gradient)
export(hermite)
export(homogeneous_components)
export(homogenize)
export(insert)
export(is.bernstein)
export(is.bezier)
export(is.chebyshev)
export(is.constant)
export(is.homogeneous)
export(is.linear)
export(is.mpoly)
export(is.mpolyList)
export(is.unipoly)
export(is.wholenumber)
export(jacobi)
export(laguerre)
export(legendre)
export(lissajous)
export(make_indeterminate_list)
export(monomials)
export(mp)
export(mpoly)
export(mpolyList)
export(multideg)
export(normalize_coefficients)
export(partitions)
export(permutations)
export(plug)
export(solve_unipoly)
export(swap)
export(totaldeg)
export(tuples)
export(vars)
importFrom(ggplot2,aes)
importFrom(ggplot2,ggplot)
importFrom(graphics,contour)
importFrom(graphics,par)
importFrom(orthopolynom,chebyshev.c.polynomials)
importFrom(orthopolynom,chebyshev.s.polynomials)
importFrom(orthopolynom,chebyshev.t.polynomials)
importFrom(orthopolynom,chebyshev.u.polynomials)
importFrom(orthopolynom,glaguerre.polynomials)
importFrom(orthopolynom,hermite.h.polynomials)
importFrom(orthopolynom,hermite.he.polynomials)
importFrom(orthopolynom,jacobi.g.polynomials)
importFrom(orthopolynom,jacobi.p.polynomials)
importFrom(orthopolynom,legendre.polynomials)
importFrom(partitions,parts)
importFrom(plyr,ddply)
importFrom(polynom,polynomial)
importFrom(stats,coef)
importFrom(stats,deriv)
importFrom(stats,reorder)
importFrom(stringi,stri_c)
importFrom(stringi,stri_length)
importFrom(stringi,stri_replace_all_fixed)
importFrom(stringi,stri_replace_all_regex)
importFrom(stringi,stri_sub)
importFrom(stringr,"str_sub<-")
importFrom(stringr,fixed)
importFrom(stringr,str_c)
importFrom(stringr,str_count)
importFrom(stringr,str_detect)
importFrom(stringr,str_dup)
importFrom(stringr,str_extract)
importFrom(stringr,str_extract_all)
importFrom(stringr,str_locate)
importFrom(stringr,str_locate_all)
importFrom(stringr,str_remove)
importFrom(stringr,str_remove_all)
importFrom(stringr,str_replace)
importFrom(stringr,str_replace_all)
importFrom(stringr,str_split)
importFrom(stringr,str_sub)
importFrom(stringr,str_trim)
importFrom(tidyr,gather)
