#' Defunct mpoly functions
#'
#' This is a list of past functions of the mpoly package.
#'
#' The following are defunct mpoly functions:
#'
#' \itemize{
#'
#' \item grobner
#'
#' }
#'
#' @name mpoly-defunct
NULL
